
-- Versatile Ada Genetic Algorithm Software system software component.
-- Max Edwards, Department of Computer Science, University of York, UK.
-- Email: me-b@minster.york.ac.uk
-- 
-- Copyright notice: This software is (C) Max Edwards 1995.  It may be
-- freely used and distributed provided that this message is not altered,
-- except that its use in military applications is not permitted.
-- Package name: GENETIC_ALGORITHM
-- Description: Provides a framework for the construction of genetic
-- algorithms, with facilities for customising the algorithm, intended to allow
-- experimentation with different methods.
-- It is a generic package which is intended to be instantiated for a
-- particular application, and provides a population initializiation procedure
-- and the main procedure ADVANCE which breeds a new generation.

with GENETIC, TEXT_IO, RANDOM_NUMBERS, REPORT;
use GENETIC, TEXT_IO, RANDOM_NUMBERS, REPORT;

generic

  -- This package must be instantiated for a particular problem; it uses
  -- the general data-types and some of its own.

  type CHROMOSOME is private;
  with function RANDOM_CHROMOSOME return CHROMOSOME;
  with procedure CROSSOVER(
    PARENT1, PARENT2 : in CHROMOSOME; CHILD1, CHILD2 : out CHROMOSOME);
  with procedure MUTATE(
    CHROM : in out CHROMOSOME; MRATE : in PROBABILITY; MTOT : in out INTEGER);
  with procedure GET_FITNESS(
    CHROM : in out CHROMOSOME; FIT : out FITNESS_VALUE);
  with function SELECTION(STATS : STATISTICS) return PARENT_LIST;
  with procedure SCALE(STATS : in out STATISTICS);
package GENETIC_ALGORITHM is
  use INT_IO, FLO_IO;

  -- Error numbers for external procedures:

  FITNESS_ERROR : constant INTEGER := 1;
  CROSSOVER_ERROR : constant INTEGER := 2;
  MUTATION_ERROR : constant INTEGER := 3;
  SELECTION_ERROR : constant INTEGER := 4;
  SCALING_ERROR : constant INTEGER := 5;
  RANDOM_ERROR : constant INTEGER := 6;
  GRAPHICS_ERROR : constant INTEGER := 7;
  EXCEPT : INTEGER := 0;

  -- Type declarations:

  type CHROMOSOME_LIST is array(INTEGER range <>) of CHROMOSOME;
  type POPULATION(POP_SIZE : POP_RANGE; HIST_MODE : HISTORY_MODE) is
    record
      CHROMOSOMES : CHROMOSOME_LIST(1 .. POP_SIZE);
      BEST_EVER_CHROMOSOME : CHROMOSOME;
      PARAMS : RUN_PARAMETERS;
      STATS : STATISTICS(POP_SIZE, HIST_MODE);
    end record;

  -- Procedures:

  procedure INITIALIZE_FITNESSES(THE_POP : in out POPULATION);

  procedure RESET_RUN(THE_POP : in out POPULATION);

  procedure RANDOMIZE_POPULATION(THE_POP : in out POPULATION);

  procedure ADVANCE(THE_POP : in out POPULATION);

  procedure DO_RUN(THE_POP : in out POPULATION);

  procedure RANDOM_SEARCH(
    NUM_FF_CALLS : in INTEGER; CHROM : out CHROMOSOME; FIT : out FITNESS_VALUE
    );

end GENETIC_ALGORITHM;

