
-- Versatile Ada Genetic Algorithm Software system software component.
-- Max Edwards, Department of Computer Science, University of York, UK.
-- Email: me-b@minster.york.ac.uk
-- 
-- Copyright notice: This software is (C) Max Edwards 1995.  It may be
-- freely used and distributed provided that this message is not altered,
-- except that its use in military applications is not permitted.
-- Package name: GENETIC
-- Description: constant, type and global variable declarations, and some basic
-- useful procedures, for use by the GENETIC_ALGORITHM and associated packages.

with TEXT_IO, UNCHECKED_DEALLOCATION;
use TEXT_IO;

package GENETIC is

-- Commonly needed output packages:

  package INT_IO is new TEXT_IO.INTEGER_IO(INTEGER);

  package FLO_IO is new TEXT_IO.FLOAT_IO(FLOAT);

  -- Global constants:

  MAX_POP_SIZE : constant INTEGER := 2504;

  -- Global variables:

  TRACE : BOOLEAN := FALSE;

  -- Type Declarations:

  subtype PROBABILITY is FLOAT range 0.0 .. 1.0;
  subtype FITNESS_VALUE is FLOAT range 0.0 .. FLOAT'LAST;
  subtype POP_RANGE is INTEGER range 2 .. MAX_POP_SIZE;
  type RUN_PARAMETERS is
    record
      POPULATION_SIZE : POP_RANGE := 70;
      CROSSOVER_RATE : PROBABILITY := 0.75;
      MUTATION_RATE : PROBABILITY := 0.015;
      NUM_GENS : INTEGER := 127;
      REPORT_RATE : INTEGER := -1;
      SEED_RATE : INTEGER := 0;
      SEED_SCOPE : PROBABILITY := 0.0;
    end record;
  type FITNESS_RECORD;
  type FITNESS_LIST is access FITNESS_RECORD;
  type FITNESS_RECORD is
    record
      MAX_RAW_FITNESS, MIN_RAW_FITNESS, AVG_RAW_FITNESS : FITNESS_VALUE := 0.0
        ;
      ON_LINE_PERFORMANCE, OFF_LINE_PERFORMANCE : FITNESS_VALUE := 0.0;
      NEW_BEST_EVER : BOOLEAN := FALSE;
      NEXT : FITNESS_LIST := null;
    end record;
  type FITNESS_ARRAY is array(INTEGER range <>) of FITNESS_VALUE;
  type PARENT_LIST is array(INTEGER range <>) of INTEGER;
  type HISTORY_MODE is (NO_HISTORY, KEEP_HISTORY);

  -- The main type structure:

  type STATISTICS(POP_SIZE : POP_RANGE; HIST_MODE : HISTORY_MODE) is
    record
      RAW_FITNESSES : FITNESS_ARRAY(1 .. POP_SIZE);
      SCALED_FITNESSES : FITNESS_ARRAY(1 .. POP_SIZE);
      SUM_SCALED_FITNESS : FITNESS_VALUE := 0.0;
      MOST_FIT, LEAST_FIT, GENERATIONS, CROSSOVERS, MUTATIONS, CLONES :
        INTEGER := 0;
      FITNESS_HISTORY : FITNESS_RECORD;
      BEST_EVER_RAW_FITNESS, WORST_EVER_RAW_FITNESS : FITNESS_VALUE := 0.0;
    end record;

  -- General procedures manipulating this data-type:

  procedure CALCULATE_STATS(STATS : in out STATISTICS);

  function CONVERGED(STATS : STATISTICS) return BOOLEAN;

  procedure FREE_MEMORY(STATS : in out STATISTICS);

end GENETIC;

