
-- Versatile Ada Genetic Algorithm Software system software component.
-- Max Edwards, Department of Computer Science, University of York, UK.
-- Email: me-b@minster.york.ac.uk
-- 
-- Copyright notice: This software is (C) Max Edwards 1995.  It may be
-- freely used and distributed provided that this message is not altered,
-- except that its use in military applications is not permitted.
-- Package name : META_PACK
-- Description: provides a Meta-level fitness function, which evaluates a given
-- parameter set by running a genetic algorithm with those parameters using the
-- given application-fitness function.  Provides the relevant parameter-set type
-- (META_PHENOTYPE), a standard chromosome package (META_CHROM), and the required
-- META_CHROM.CHROMOSOME -> META_PHENOTYPE decoding routine.  See example program
-- for use of this package as a parameter-set optimiser using a high-order
-- genetic algorithm

with TEXT_IO, GENETIC_ALGORITHM, GENETIC, STANDARD_CHROMOSOME, REPORT;
use TEXT_IO, GENETIC;

generic
  TEST_RUNS : INTEGER := 1;
  PARAM_BITS : INTEGER := 6;
  MAX_FF_CALLS : INTEGER := 2500;
  MAX_APP_GENS : INTEGER := 256;
  type APP_CHROMOSOME is private;
  with procedure INITIALIZE_PROBLEM;
  with procedure APP_FITNESS(
    CHROM : in out APP_CHROMOSOME; FIT : out FITNESS_VALUE);
  with function RANDOM_APP_CHROMOSOME return APP_CHROMOSOME;
  with function SELECT1(STATS : in STATISTICS) return PARENT_LIST;
  with function SELECT2(STATS : in STATISTICS) return PARENT_LIST;
  with function SELECT3(STATS : in STATISTICS) return PARENT_LIST;
  with function SELECT4(STATS : in STATISTICS) return PARENT_LIST;
  with procedure CROSSOVER1(
    PARENT1, PARENT2 : in APP_CHROMOSOME; CHILD1, CHILD2 : out APP_CHROMOSOME)
    ;
  with procedure CROSSOVER2(
    PARENT1, PARENT2 : in APP_CHROMOSOME; CHILD1, CHILD2 : out APP_CHROMOSOME)
    ;
  with procedure CROSSOVER3(
    PARENT1, PARENT2 : in APP_CHROMOSOME; CHILD1, CHILD2 : out APP_CHROMOSOME)
    ;
  with procedure CROSSOVER4(
    PARENT1, PARENT2 : in APP_CHROMOSOME; CHILD1, CHILD2 : out APP_CHROMOSOME)
    ;
  with procedure MUTATE1(CHROM : in out APP_CHROMOSOME;
                         MRATE : in PROBABILITY; MTOT : in out INTEGER);
  with procedure MUTATE2(CHROM : in out APP_CHROMOSOME;
                         MRATE : in PROBABILITY; MTOT : in out INTEGER);
  with procedure MUTATE3(CHROM : in out APP_CHROMOSOME;
                         MRATE : in PROBABILITY; MTOT : in out INTEGER);
  with procedure MUTATE4(CHROM : in out APP_CHROMOSOME;
                         MRATE : in PROBABILITY; MTOT : in out INTEGER);
  with procedure SCALE1(STATS : in out STATISTICS);
  with procedure SCALE2(STATS : in out STATISTICS);
  with procedure SCALE3(STATS : in out STATISTICS);
  with procedure SCALE4(STATS : in out STATISTICS);
package META_PACK is
  use INT_IO, FLO_IO;

  -- Constants:

  PMAX : constant FLOAT := FLOAT(2 ** PARAM_BITS - 1);

  -- Type declarations:

  type META_PHENOTYPE is
    record
      XRATE, MRATE : PROBABILITY := 0.0;
      NUMGENS, CROSS_MODE, MUT_MODE, SCALE_MODE, SELECT_MODE, POPSIZE :
        INTEGER := -1;
    end record;

  -- Procedures:

  procedure NO_INIT_PROBLEM;

  procedure PUT(R : in META_PHENOTYPE);

  procedure STATE_PARAMETERS;

  package META_CHROM is new STANDARD_CHROMOSOME(2, 3 * PARAM_BITS + 8);

  function META_DECODE(CHROM : META_CHROM.CHROMOSOME) return META_PHENOTYPE;

  procedure OFF_LINE_FITNESS(
    CHROM : in out META_CHROM.CHROMOSOME; FIT : out FITNESS_VALUE);

  procedure ON_LINE_FITNESS(
    CHROM : in out META_CHROM.CHROMOSOME; FIT : out FITNESS_VALUE);

end META_PACK;

