
-- Versatile Ada Genetic Algorithm Software system software component.
-- Max Edwards, Department of Computer Science, University of York, UK.
-- Email: me-b@minster.york.ac.uk
-- 
-- Copyright notice: This software is (C) Max Edwards 1995.  It may be
-- freely used and distributed provided that this message is not altered,
-- except that its use in military applications is not permitted.
-- Package name: PERMUTATION_CHROMOSOME
-- Description : A generic package which provides everything required,
-- except of course the application fitness function, to instantiate the
-- GENETIC.ALGORITHM package for use with permutation chromosomes, i.e. those
-- which use each gene value exactly once.

with RANDOM_NUMBERS, TEXT_IO, GENETIC;
use RANDOM_NUMBERS, TEXT_IO, GENETIC;

generic
  CHROM_LENGTH : INTEGER;
package PERMUTATION_CHROMOSOME is
  use INT_IO, FLO_IO;
  subtype GENE is INTEGER range 1 .. CHROM_LENGTH;
  type CHROMOSOME is array(1 .. CHROM_LENGTH) of GENE;

  -- Useful routines for chromosome manipulation:

  procedure PUT(CHROM : in CHROMOSOME);

  -- Prints the given chromosome as a string of numbers

  function VALID(CHROM : CHROMOSOME) return BOOLEAN;

  -- Confirms that the given chromosome is a true permutation
  -- Provide a random chromsome which is a permutation string:

  function RANDOM_CHROMOSOME return CHROMOSOME;

  procedure PMX1_CROSSOVER(
    PARENT1, PARENT2 : in CHROMOSOME; CHILD1, CHILD2 : out CHROMOSOME);

  -- Partially matched crossover for breeding permutations.
  -- Original method - due to David Goldberg (in Goldberg 1989).

  procedure PMX2_CROSSOVER(
    PARENT1, PARENT2 : in CHROMOSOME; CHILD1, CHILD2 : out CHROMOSOME);

  -- Partially matched crossover for breeding permutations.
  -- Aligned first parental gene method. (one of the parents is rotated
  -- until both have the same first gene value, in the hope that this will
  -- create offspring which bear more resemblance to the parents than in the
  -- original PMX method.

  procedure ORDER_CROSSOVER(
    PARENT1, PARENT2 : in CHROMOSOME; CHILD1, CHILD2 : out CHROMOSOME);

  -- Another permutation crossover method, which seems to be more effective
  -- than Goldberg's PMX
  -- See Goldberg P74

  procedure CYCLE_CROSSOVER(
    PARENT1, PARENT2 : in CHROMOSOME; CHILD1, CHILD2 : out CHROMOSOME);

  -- Again, this method seems to be better than the standard PMX algorithm
  -- See Goldberg P74

  procedure SWAP_MUTATE(
    CHROM : in out CHROMOSOME; MRATE : in PROBABILITY; MTOT : in out INTEGER);

  -- Simple permutation mutation; swaps two distinct genes.

  procedure SHIFT_MUTATE(
    CHROM : in out CHROMOSOME; MRATE : in PROBABILITY; MTOT : in out INTEGER);

  procedure INVERT_MUTATE(
    CHROM : in out CHROMOSOME; MRATE : in PROBABILITY; MTOT : in out INTEGER);

  procedure SCRAMBLE_MUTATE(
    CHROM : in out CHROMOSOME; MRATE : in PROBABILITY; MTOT : in out INTEGER);

end PERMUTATION_CHROMOSOME;

